@@ -1,44 +1,15 @@
 #ifndef __divonne_rule_h__
 #define __divonne_rule_h__
-//Compilation note for R interface: move into a .h
-/*
-	Rule.c
-		integration with cubature rules
-		code lifted with minor modifications from DCUHRE
-		by J. Berntsen, T. Espelid, and A. Genz
-		this file is part of Divonne
-		last modified 9 Feb 05 th
-*/
-
+#include "common_stddecl.h"
+#include "divonne_decl.h"
+#include "divonne_util.h"
 
+/*********************************************************************/
 enum { nrules = 5 };
 
-#define TYPEDEFSET \
-  typedef struct { \
-    count n; \
-    real weight[nrules], scale[nrules], norm[nrules]; \
-    real gen[NDIM]; \
-  } Set
-
-/*********************************************************************/
-
-static inline void RuleIni(Rule *rule)
+static void divonneRule13Alloc(Rule *rule)
 {
-  rule->first = NULL;
-}
-
-/*********************************************************************/
-
-static inline void RuleFree(Rule *rule)
-{
-  if( rule->first ) free(rule->first);
-}
-
-/*********************************************************************/
-
-static void Rule13Alloc(Rule *rule)
-{
-  static creal w[][nrules] = {
+  static ctreal w[][nrules] = {
     { .00844923090033615,     .3213775489050763,     .3372900883288987,
      -.8264123822525677,      .6539094339575232 },
     { .023771474018994404,   -.1767341636743844,    -.1644903060344491,
@@ -69,7 +40,7 @@ static void Rule13Alloc(Rule *rule)
       .003760268580063992,    .02561989142123099 }
   };
 
-  static creal g[] = {
+  static ctreal g[] = {
      .12585646717265545,      .3506966822267133,
      .4795480315809981,       .4978005239276064,
      .25,                     .07972723291487795,
@@ -81,7 +52,7 @@ static void Rule13Alloc(Rule *rule)
 
   enum { nsets = 14, ndim = 2 };
 
-  TYPEDEFSET;
+  DIVONNETYPEDEFSET;
 
   count n, r;
   Set *first, *last, *s, *t;
@@ -175,7 +146,7 @@ static void Rule13Alloc(Rule *rule)
 
   for( s = first; s <= last; ++s )
     for( r = 1; r < nrules - 1; ++r ) {
-      creal scale = (s->weight[r] == 0) ? 100 :
+      ctreal scale = (s->weight[r] == 0) ? 100 :
         -s->weight[r + 1]/s->weight[r];
       real sum = 0;
       for( t = first; t <= last; ++t )
@@ -187,9 +158,9 @@ static void Rule13Alloc(Rule *rule)
 
 /*********************************************************************/
 
-static void Rule11Alloc(Rule *rule)
+static void divonneRule11Alloc(Rule *rule)
 {
-  static creal w[][nrules] = {
+  static ctreal w[][nrules] = {
     { .0009903847688882167,  1.715006248224684,     1.936014978949526,
       .517082819560576,      2.05440450381852 },
     { .0084964717409851,     -.3755893815889209,    -.3673449403754268,
@@ -218,7 +189,7 @@ static void Rule11Alloc(Rule *rule)
       .003019236275367777,    .0028782064230998723 }
   };
 
-  static creal g[] = {
+  static ctreal g[] = {
      .095,                    .25,
      .375,                    .4,
      .4975,                   .49936724991757,
@@ -229,7 +200,7 @@ static void Rule11Alloc(Rule *rule)
 
   enum { nsets = 13, ndim = 3 };
 
-  TYPEDEFSET;
+  DIVONNETYPEDEFSET;
 
   count n, r;
   Set *first, *last, *s, *t;
@@ -322,7 +293,7 @@ static void Rule11Alloc(Rule *rule)
 
   for( s = first; s <= last; ++s )
     for( r = 1; r < nrules - 1; ++r ) {
-      creal scale = (s->weight[r] == 0) ? 100 :
+      ctreal scale = (s->weight[r] == 0) ? 100 :
         -s->weight[r + 1]/s->weight[r];
       real sum = 0;
       for( t = first; t <= last; ++t )
@@ -334,9 +305,9 @@ static void Rule11Alloc(Rule *rule)
 
 /*********************************************************************/
 
-static void Rule9Alloc(Rule *rule)
+static void divonneRule9Alloc(Rule *rule)
 {
-  static creal w[] = {
+  static ctreal w[] = {
     -.0023611709677855117884,   .11415390023857325268,
     -.63833920076702389094,     .74849988504685208004,
     -.0014324017033399125142,   .057471507864489725949,
@@ -359,14 +330,14 @@ static void Rule9Alloc(Rule *rule)
     -.32544759695960125297,     .0017708782258391338413,
      .0010743012775049343856,   .25150011495314791996 };
 
-  static creal g[] = {
+  static ctreal g[] = {
      .47795365790226950619,     .20302858736911986780,
      .44762735462617812882,     .125,
      .34303789878087814570 };
 
   enum { nsets = 9 };
 
-  TYPEDEFSET;
+  DIVONNETYPEDEFSET;
 
   ccount ndim = ndim_;
   ccount twondim = 1 << ndim;
@@ -466,7 +437,7 @@ static void Rule9Alloc(Rule *rule)
 
   for( s = first; s <= last; ++s )
     for( r = 1; r < nrules - 1; ++r ) {
-      creal scale = (s->weight[r] == 0) ? 100 :
+      ctreal scale = (s->weight[r] == 0) ? 100 :
         -s->weight[r + 1]/s->weight[r];
       real sum = 0;
       for( t = first; t <= last; ++t )
@@ -478,9 +449,9 @@ static void Rule9Alloc(Rule *rule)
 
 /*********************************************************************/
 
-static void Rule7Alloc(Rule *rule)
+static void divonneRule7Alloc(Rule *rule)
 {
-  static creal w[] = {
+  static ctreal w[] = {
      .019417866674748388428,   -.40385257701150182546,
      .64485668767465982223,     .01177982690775806141,
     -.18041318740733609012,    -.088785828081335044443,
@@ -493,13 +464,13 @@ static void Rule7Alloc(Rule *rule)
      .0058899134538790307051,  -.0048544666686870971071,
      .35514331232534017777 };
 
-  static creal g[] = {
+  static ctreal g[] = {
      .47795365790226950619,     .20302858736911986780,
      .375,                      .34303789878087814570 };
 
   enum { nsets = 6 };
 
-  TYPEDEFSET;
+  DIVONNETYPEDEFSET;
 
   ccount ndim = ndim_;
   ccount twondim = 1 << ndim;
@@ -569,7 +540,7 @@ static void Rule7Alloc(Rule *rule)
 
   for( s = first; s <= last; ++s )
     for( r = 1; r < nrules - 1; ++r ) {
-      creal scale = (s->weight[r] == 0) ? 100 :
+      ctreal scale = (s->weight[r] == 0) ? 100 :
         -s->weight[r + 1]/s->weight[r];
       real sum = 0;
       for( t = first; t <= last; ++t )
@@ -578,107 +549,4 @@ static void Rule7Alloc(Rule *rule)
       s->norm[r] = 1/sum;
     }
 }
-
-/*********************************************************************/
-
-static real *ExpandFS(cBounds *b, real *g, real *x)
-{
-  count dim, ndim = ndim_;
-
-next:
-  /* Compute centrally symmetric sum for permutation of G */
-
-  for( dim = 0; dim < ndim; ++dim )
-    *x++ = (.5 + g[dim])*b[dim].lower + (.5 - g[dim])*b[dim].upper;
-
-  for( dim = 0; dim < ndim; ) {
-    g[dim] = -g[dim];
-    if( g[dim++] < 0 ) goto next;
-  }
-
-  /* Find next distinct permutation of G and loop back for next sum.
-     Permutations are generated in reverse lexicographic order. */
-
-  for( dim = 1; dim < ndim; ++dim ) {
-    creal gd = g[dim];
-    count big = dim - 1;
-    if( g[big] > gd ) {
-      count i, j = dim, lastbig = big;
-      for( i = 0; i < --j; ++i ) {
-        creal tmp = g[i];
-        g[i] = g[j];
-        g[j] = tmp;
-        if( tmp <= gd ) --big;
-        if( g[i] > gd ) lastbig = i;
-      }
-      if( g[big] <= gd ) big = lastbig;
-      g[dim] = g[big];
-      g[big] = gd;
-      goto next;
-    }
-  }
-
-  /* Restore original order to generators */
-
-  for( dim = 0; dim < --ndim; ++dim ) {
-    creal tmp = g[dim];
-    g[dim] = g[ndim];
-    g[ndim] = tmp;
-  }
-
-  return x;
-}
-
-/*********************************************************************/
-
-static void SampleRule(cSamples *samples, cBounds *b, creal vol)
-{
-  TYPEDEFSET;
-
-  real *x = samples->x, *f = samples->f;
-  Set *first = (Set *)samples->rule->first;
-  Set *last = (Set *)samples->rule->last;
-  Set *s;
-  creal *errcoeff = samples->rule->errcoeff;
-  count comp, rul, n;
-
-  for( s = first; s <= last; ++s )
-    if( s->n ) x = ExpandFS(b, s->gen, x);
-
-  DoSample(samples->n, ndim_, samples->x, f);
-
-  for( comp = 0; comp < ncomp_; ++comp ) {
-    real sum[nrules];
-    creal *f1 = f++;
-
-    Zap(sum);
-    for( s = first; s <= last; ++s )
-      for( n = s->n; n; --n ) {
-        creal fun = *f1;
-        f1 += ncomp_;
-        for( rul = 0; rul < nrules; ++rul )
-          sum[rul] += fun*s->weight[rul];
-      }
-
-    /* Search for the null rule, in the linear space spanned by two
-       successive null rules in our sequence, which gives the greatest
-       error estimate among all normalized (1-norm) null rules in this
-       space. */
-
-    for( rul = 1; rul < nrules - 1; ++rul ) {
-      real maxerr = 0;
-      for( s = first; s <= last; ++s )
-        maxerr = Max(maxerr,
-          fabs(sum[rul + 1] + s->scale[rul]*sum[rul])*s->norm[rul]);
-      sum[rul] = maxerr;
-    }
-
-    samples->avg[comp] = vol*sum[0];
-    samples->err[comp] = vol*(
-      (errcoeff[0]*sum[1] <= sum[2] && errcoeff[0]*sum[2] <= sum[3]) ?
-        errcoeff[1]*sum[1] :
-        errcoeff[2]*Max(Max(sum[1], sum[2]), sum[3]));
-  }
-}
-
 #endif
